/*	This file is part of the software similarity tester SIM.
	Written by Dick Grune, Vrije Universiteit, Amsterdam.
	$Id: compare.h,v 1.2 1998/01/21 14:27:47 dick Exp $
*/

/*	Compares each new text to the appropriate texts.
	Stores the runs found in the AISO heap.
	Runs contain references to positions in the input files.
*/

extern void Compare(void);
