unit urhlp14;

{$ifdef fpc}
  {$mode delphi}
{$endif}

interface

type
  TTest = record
  strict private
    Test1: Integer;
  private
    Test2: Integer;
  public
    Test3: Integer;
  end;

implementation

end.
