{ X11 extensions we want to enable at compile time }
{$DEFINE ENABLE_X11_EXTENSION_XRANDR}
{$DEFINE ENABLE_X11_EXTENSION_XF86VIDMODE}
{$DEFINE ENABLE_X11_EXTENSION_XF86DGA1}
{$DEFINE ENABLE_X11_EXTENSION_XF86DGA2}
{$DEFINE ENABLE_X11_EXTENSION_XSHM}
{$DEFINE ENABLE_X11_EXTENSION_GLX}
{$DEFINE ENABLE_X11_EXTENSION_XINPUT2}
