
{$IFDEF OGC_INTERFACE}
const
  LWP_STATES_READY = $00000000;
  LWP_STATES_DORMANT = $00000001;
  LWP_STATES_SUSPENDED = $00000002;
  LWP_STATES_TRANSIENT = $00000004;
  LWP_STATES_DELAYING = $00000008;
  LWP_STATES_WAITING_FOR_TIME = $00000010;
  LWP_STATES_WAITING_FOR_BUFFER = $00000020;
  LWP_STATES_WAITING_FOR_SEGMENT = $00000040;
  LWP_STATES_WAITING_FOR_MESSAGE = $00000080;
  LWP_STATES_WAITING_FOR_EVENT = $00000100;
  LWP_STATES_WAITING_FOR_MUTEX = $00000200;
  LWP_STATES_WAITING_FOR_SEMAPHORE = $00000400;
  LWP_STATES_WAITING_FOR_CONDVAR = $00000800;
  LWP_STATES_WAITING_FOR_JOINATEXIT = $00001000;
  LWP_STATES_WAITING_FOR_RPCREPLAY = $00002000;
  LWP_STATES_WAITING_FOR_PERIOD = $00004000;
  LWP_STATES_WAITING_FOR_SIGNAL = $00008000;
  LWP_STATES_INTERRUPTIBLE_BY_SIGNAL = $00010000;
  LWP_STATES_LOCALLY_BLOCKED = ( LWP_STATES_WAITING_FOR_BUFFER or LWP_STATES_WAITING_FOR_SEGMENT or LWP_STATES_WAITING_FOR_MESSAGE or LWP_STATES_WAITING_FOR_SEMAPHORE or LWP_STATES_WAITING_FOR_MUTEX or LWP_STATES_WAITING_FOR_CONDVAR or LWP_STATES_WAITING_FOR_JOINATEXIT or LWP_STATES_WAITING_FOR_SIGNAL );
  LWP_STATES_WAITING_ON_THREADQ = ( LWP_STATES_LOCALLY_BLOCKED or LWP_STATES_WAITING_FOR_RPCREPLAY );
  LWP_STATES_BLOCKED = ( LWP_STATES_DELAYING or LWP_STATES_WAITING_FOR_TIME or LWP_STATES_WAITING_FOR_PERIOD or LWP_STATES_WAITING_FOR_EVENT or LWP_STATES_WAITING_ON_THREADQ or LWP_STATES_INTERRUPTIBLE_BY_SIGNAL );
  //#ifdef LIBOGC_INTERNAL
  //#include <libogc/lwp_states.inl>
  //#endif
{$ENDIF}

