{$include dxegen/fpmake.pp}

{$include fpcm/fpmake.pp}

{$include fpcmkcfg/fpmake.pp}

{$include fpcres/fpmake.pp}

{$include fpcreslipo/fpmake.pp}

{$include fpdoc/fpmake.pp}

{$include fpmc/fpmake.pp}

{$include fppkg/fpmake.pp}

{$include fprcp/fpmake.pp}

{$include h2pas/fpmake.pp}

{$include importtl/fpmake.pp}

{$include instantfpc/fpmake.pp}

{$include pas2fpm/fpmake.pp}

{$include pas2jni/fpmake.pp}

{$include pas2ut/fpmake.pp}

{$include rmwait/fpmake.pp}

{$include tply/fpmake.pp}

{$include unicode/fpmake.pp}

