const
  IDOK = 1;

  IDI_SYSINFO       = 100;
  IDC_SYSINFO       = 101;
  IDM_MENU          = 102;
  IDD_ABOUTBOX      = 103;
  IDR_MENUBAR       = 104;
  IDS_HELP          = 105;
  IDS_COMMAND1      = 301;
  IDM_MAIN_COMMAND1 = 40001;
  IDM_HELP_ABOUT    = 40003;
  IDM_QUIT          = 40004;
  IDM_MENUITEM40005 = 40005;
  IDM_MENUITEM40006 = 40006;

