public class tjavalowercaseproc {

public static void main(String[] args)
{
  org.freepascal.test.lcproc.tc c;

  org.freepascal.test.lcproc.tlowercaseproc.doIt();

  c = new org.freepascal.test.lcproc.tc();
  c.methodName();
  org.freepascal.test.lcproc.tc.classMethodName();
  c.xToY();
  c.prefixThingToTest();
  c.rc64Encode();
}

}
