// CSupport.h - protos for external C functions called from pascal

#ifndef CSupport_included
#define CSupport_included

#ifdef __cplusplus
extern "C" {
#endif

SInt32 CFunction(void);
SInt32 CPlusFunction(void);

#ifdef __cplusplus
}
#endif

#endif // CSupport_included
