{
 QPasHello.pas

 *****************************************************************************
 *                                                                           *
 *  This demonstration program is public domain, which means no copyright,   *
 * but also no warranty!                                                     *
 *                                                                           *
 *  This program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *                                                                           *
 *****************************************************************************

 This application will create a popup message 'My Hello World' and then quit

 The message will last a few seconds.

 Author: Felipe Monteiro de Carvalho
}
program QPasHello;

uses symbian;

begin
  User_InfoPrint('My Hello World');
end.

