unit uw13583;

interface

type
  TEDBString = WideString;
  TEDBString2 = ansiString;

const
  DEFAULT_SIGNATURE = TEDBString('edb_signature');
  DEFAULT_SIGNATURE2 = TEDBString('edb_signature2');

implementation

end.
