#include "Carbon.r"

resource 'STR#' (128, "Messages", purgeable) {
        {       /* array StringArray: 1 elements */
                /* [1] */
                "Hello, I'm C, calling pascal main to display this."
        }
};
