{ %fail }

{ no longer supported by current Delphi versions and doesn't mean anything
  anyway }

{ Source provided for Free Pascal Bug Report 2650 }
{ Submitted by "marcov" on  2003-08-21 }
{ e-mail: marco@freepascal.org }

{$mode Delphi}

type someclass=class
                 property integer :integer;
                 end;

begin
end.
