{ Strings }
sVideoFailed = 0; {Video initialization failed.}
sButtonDefault = 1; { Button default }
sButtonDisabled = 2; { Button disabled }
sButtonNormal = 3; { Button normal }
sButtonSelected = 4; { Button selected }
sButtonShadow = 5; { Button shadow }
sButtonShortcut = 6; { Button shortcut }
sChangeDirectory = 7; { Change Directory }
sClipboard = 8; { Clipboard }
sClusterNormal = 9; { Cluster normal }
sClusterSelected = 10; { Cluster selected }
sClusterShortcut = 11; { Cluster shortcut }
sColor = 12; { Color }
sColors = 13; { Colors }
sConfirm = 14; { Confirm }
sDeleteFile = 15; { Delete file?#13#10#13#3%s }
sDirectory = 16; { Directory }
sDisabled = 17; { Disabled }
sDrives = 18; { Drives }
sError = 19; { Error }
sFileAlreadyOpen = 20; { #3%s#13#10#13#3is already open in window %d. }
sFileCreateError = 21; { Error creating file %s }
sFileReadError = 22; { Error reading file %s }
sFileUntitled = 23; { Save untitled file? }
sFileWriteError = 24; { Error writing to file %s }
sFind = 25; { Find }
sFrameActive = 26; { Frame active }
sFrameBackground = 27; { Frame/background }
sFrameIcons = 28; { Frame icons }
sFramePassive = 29; { Frame passive }
sHighlight = 30; { Highlight }
sHistoryBarIcons = 31; { History bar icons }
sHistoryBarPage = 32; { History bar page }
sHistoryButton = 33; { History button }
sHistorySides = 34; { History sides }
sInformation = 35; { Information }
sInformationPane = 36; { Information pane }
sInputArrow = 37; { Input arrow }
sInputNormal = 38; { Input normal }
sInputSelected = 39; { Input selected }
sInvalidCharacter = 40; { Invalid character in input }
sInvalidDirectory = 41; { Invalid directory. }
sInvalidDriveOrDir = 42; { Invalid drive or directory. }
sInvalidFileName = 43; { Invalid file name. }
sInvalidPicture = 44; { Input does not conform to picture: %s }
sInvalidValue = 45; { Value not in the range %d to %d }
sInverse = 46; { Inverse }
sJumpTo = 47; { Jump To }
sLabelNormal = 48; { Label normal }
sLabelSelected = 49; { Label selected }
sLabelShortcut = 50; { Label shortcut }
sListDivider = 51; { List divider }
sListFocused = 52; { List focused }
sListNormal = 53; { List normal }
sListSelected = 54; { List selected }
sModified = 55; { #3%s#13#10#13#3has been modified.  Save? }
sNoName = 56; { NONAME }
sNormal = 57; { Normal }
sNormalText = 58; { Normal text }
sNotInList = 59; { Input not in valid-list }
sOpen = 60; { Open }
sOutOfMemory = 61; { Not enough memory for this operation. }
sOutOfUnNamedWindows = 62; { Out of unnamed window numbers.  Save or discard some unnamed files and try again. }
sPasteNotPossible = 63; { Wordwrap on:  Paste not possible in current margins when at end of line. }
sReformatDocument = 64; { Reformat Document }
sReformatNotPossible = 65; { Paragraph reformat not possible while trying to wrap current line with current margins. }
sReformattingTheDocument = 66; { Reformatting the document: }
sReplace = 67; { Replace }
sReplaceFile = 68; { Replace file?#13#10#13#3%s }
sReplaceNotPossible = 69; { Wordwrap on:  Replace not possible in current margins when at end of line. }
sReplaceThisOccurence = 70; { Replace this occurence? }
sRightMargin = 71; { Right Margin }
sSaveAs = 72; { Save As }
sScrollbarIcons = 73; { Scroll bar icons }
sScrollbarPage = 74; { Scroll bar page }
sSearchStringNotFound = 75; { Search string not found. }
sSelectFormatStart = 76; { Select Format Start }
sSelectWhereToBegin = 77; { Please select where to begin. }
sSelected = 78; { Selected }
sSelectedDisabled = 79; { Selected disabled }
sSetting = 80; { Setting: }
sShortcut = 81; { Shortcut }
sShortcutSelected = 82; { ShortcutSelected }
sStaticText = 83; { Static text }
sTabSettings = 84; { Tab Settings }
sText = 85; { Text }
sTooManyFiles = 86; { Too many files. }
sTypeExitOnReturn = 87; { Type EXIT to return... }
sUnderline = 88; { Underline }
sUnknownDialog = 89; { Unknown dialog requested! }
sUntitled = 90; { Untitled }
sWarning = 91; { Warning }
sWindowList = 92; { Window List }
sWordWrapNotPossible = 93; { Wordwrap on:  Wordwrap not possible in current margins with continuous line. }
sWordWrapOff = 94; { You must turn on wordwrap before you can reformat. }
smApr = 95; { Apr }
smAug = 96; { Aug }
smDec = 97; { Dec }
smFeb = 98; { Feb }
smJan = 99; { Jan }
smJul = 100; { Jul }
smJun = 101; { Jun }
smMar = 102; { Mar }
smMay = 103; { May }
smNov = 104; { Nov }
smOct = 105; { Oct }
smSep = 106; { Sep }
{ Labels }
slAbout = 107;  { ~A~bout }
slAltF1 = 108;  { Alt+F1 }
slAltF3Close = 109;  { ~Alt+F3~ Close }
slAltXExit = 110; { ~Alt-X~ Exit }
slBackground = 111; { ~B~ackground }
slCancel = 112; { Cancel }
slCascade = 113; { C~a~scade }
slCaseSensitive = 114; { ~C~ase sensitive }
slChDir = 115; { ~C~hdir }
slChangeDir = 116; { ~C~hange dir... }
slClear = 117; { C~l~ear }
slClose = 118; { ~C~lose }
slCloseAll = 119; { Cl~o~se all }
slColor = 120; { ~C~olor }
slContents = 121;  { ~C~ontents }
slCopy = 122; { ~C~opy }
slCtrlF1 = 123;  { Ctrl+F1 }
slCurrentLine = 124; { ~C~urrent line }
slCut = 125; { Cu~t~ }
slDOSShell = 126; { ~D~OS shell }
slDelete = 127; { ~D~elete }
slDirectoryName = 128; { Directory ~n~ame }
slDirectoryTree = 129; { Directory ~t~ree }
slEdit = 130; { ~E~dit }
slEntireDocument = 131; { ~E~ntire document }
slExit = 132; { E~x~it }
slF10Menu = 133; { ~F10~ Menu }
slF1Help = 134; { ~F1~ Help }
slF3Open = 135; { ~F3~ Open }
slFile = 136; { ~F~ile }
slFiles = 137; { ~F~iles }
slForeground = 138; { ~F~oreground }
slGroup = 139; { ~G~roup }
slHelp = 140; { ~H~elp }
slIndex = 141;  { ~I~ndex }
slItem = 142; { ~I~tem }
slLineNumber = 143; { ~L~ine number }
slName = 144; { ~N~ame }
slNew = 145; { ~N~ew }
slNewText = 146; { ~N~ew text }
slNext = 147; { ~N~ext }
slNo = 148; { ~N~o }
slOk = 149; { O~k~ }
slOpen = 150; { ~O~pen }
slOpenDots = 151; { ~O~pen... }
slPaste = 152; { ~P~aste }
slPrevious = 153; { ~P~revious }
slPreviousTopic = 154;  { ~P~revious topic }
slPromptOnReplace = 155; { ~P~rompt on replace }
slReformatDocument = 156; { ~R~eformat document }
slReplace = 157; { ~R~eplace }
slReplaceAll = 158; { ~R~eplace all }
slRevert = 159; { ~R~evert }
slSave = 160; { ~S~ave }
slSaveAll = 161; { Save a~l~l }
slSaveAs = 162; { S~a~ve as... }
slSaveFileAs = 163; { ~S~ave file as }
slShiftF1 = 164;  { Shift+F1 }
slSizeMove = 165; { ~S~ize/Move }
slTextToFind = 166; { ~T~ext to find }
slTile = 167; { ~T~ile }
slTopicSearch = 168;  { ~T~opic search }
slUndo = 169; { ~U~ndo }
slUsingHelp = 170;  { ~U~sing help }
slWholeWordsOnly = 171; { ~W~hole words only }
slWindow = 172;  { ~W~indow }
slWindows = 173; { ~W~indows }
slYes = 174; { ~Y~es }
slZoom = 175; { ~Z~oom }
slAltF3 = 176; { Alt+F3 }
slAltX = 177; { Alt+X }
slF2 = 178; { F2 }
slF3 = 179; { F3 }
slF5 = 180; { F5 }
slF6 = 181; { F6 }
slCtrlDel = 182; { Ctrl+Del }
slCtrlF5 = 183; { Ctrl+F5 }
slCtrlIns = 184; { Ctrl+Ins }
slShiftDel = 185; { Shift+Del }
slShiftF6 = 186; { Shift+F6 }
slShiftIns = 187; { Shift+Ins }
