{ Source provided for Free Pascal Bug Report 2704 }
{ Submitted by "Johannes Berg" on  2003-10-01 }
{ e-mail: johannes -at- sipsolutions -dot- de }
program h;

{$mode delphi}

type
  TTest = class
    constructor Create; virtual; abstract;
  end;

begin
end.
