{ Parsed from Webkit }

{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}

unit AnonClassDefinitionsWebkit;

interface

type
  Protocol = objcclass external;
  NSAffineTransform = objcclass external;
  NSData = objcclass external;
  NSAppleEventDescriptor = objcclass external;
  NSAppleEventManager = objcclass external;
  NSDictionary = objcclass external;
  NSString = objcclass external;
  NSURL = objcclass external;
  NSAppleScript = objcclass external;
  NSMutableData = objcclass external;
  NSMutableDictionary = objcclass external;
  NSMutableArray = objcclass external;
  NSArchiver = objcclass external;
  NSUnarchiver = objcclass external;
  NSIndexSet = objcclass external;
  NSArray = objcclass external;
  NSAttributedString = objcclass external;
  NSMutableAttributedString = objcclass external;
  NSAutoreleasePool = objcclass external;
  NSError = objcclass external;
  NSBundle = objcclass external;
  NSCache = objcclass external;
  NSDateComponents = objcclass external;
  NSLocale = objcclass external;
  NSTimeZone = objcclass external;
  NSCalendar = objcclass external;
  NSCalendarDate = objcclass external;
  NSCharacterSet = objcclass external;
  NSMutableCharacterSet = objcclass external;
  NSClassDescription = objcclass external;
  NSCoder = objcclass external;
  NSPredicateOperator = objcclass external;
  NSExpression = objcclass external;
  NSComparisonPredicate = objcclass external;
  NSCompoundPredicate = objcclass external;
  NSDistantObject = objcclass external;
  NSException = objcclass external;
  NSPort = objcclass external;
  NSRunLoop = objcclass external;
  NSPortNameServer = objcclass external;
  NSDistantObjectRequest = objcclass external;
  NSConnection = objcclass external;
  NSPurgeableData = objcclass external;
  NSDate = objcclass external;
  NSDateFormatter = objcclass external;
  NSDecimalNumber = objcclass external;
  NSDecimalNumberHandler = objcclass external;
  NSSet = objcclass external;
  NSDistributedLock = objcclass external;
  NSDistributedNotificationCenter = objcclass external;
  NSEnumerator = objcclass external;
  NSAssertionHandler = objcclass external;
  NSPredicate = objcclass external;
  NSFileHandle = objcclass external;
  NSPipe = objcclass external;
  NSDirectoryEnumerator = objcclass external;
  NSNumber = objcclass external;
  NSFileManager = objcclass external;
  NSFormatter = objcclass external;
  NSGarbageCollector = objcclass external;
  NSHashTable = objcclass external;
  NSHost = objcclass external;
  NSHTTPCookieInternal = objcclass external;
  NSHTTPCookie = objcclass external;
  NSHTTPCookieStorageInternal = objcclass external;
  NSHTTPCookieStorage = objcclass external;
  NSIndexPath = objcclass external;
  NSMutableIndexSet = objcclass external;
  NSMethodSignature = objcclass external;
  NSInvocation = objcclass external;
  NSKeyedArchiver = objcclass external;
  NSKeyedUnarchiver = objcclass external;
  NSLock = objcclass external;
  NSConditionLock = objcclass external;
  NSRecursiveLock = objcclass external;
  NSCondition = objcclass external;
  NSMapTable = objcclass external;
  NSMetadataItem = objcclass external;
  NSMetadataQueryAttributeValueTuple = objcclass external;
  NSMetadataQueryResultGroup = objcclass external;
  NSMetadataQuery = objcclass external;
  NSInputStream = objcclass external;
  NSOutputStream = objcclass external;
  NSNetService = objcclass external;
  NSNetServiceBrowser = objcclass external;
  NSOperationQueue = objcclass external;
  NSNotification = objcclass external;
  NSNotificationCenter = objcclass external;
  NSNotificationQueue = objcclass external;
  NSNull = objcclass external;
  NSNumberFormatter = objcclass external;
  NSObject = objcclass external;
  NSScriptObjectSpecifier = objcclass external;
  NSOperation = objcclass external;
  NSBlockOperation = objcclass external;
  NSInvocationOperation = objcclass external;
  NSOrthography = objcclass external;
  NSPointerArray = objcclass external;
  NSPointerFunctions = objcclass external;
  NSPortMessage = objcclass external;
  NSMachPort = objcclass external;
  NSMessagePort = objcclass external;
  NSSocketPort = objcclass external;
  NSPortCoder = objcclass external;
  NSMachBootstrapServer = objcclass external;
  NSMessagePortNameServer = objcclass external;
  NSSocketPortNameServer = objcclass external;
  NSProcessInfo = objcclass external;
  NSPropertyListSerialization = objcclass external;
  NSProtocolChecker = objcclass external;
  NSProxy = objcclass external;
  NSTimer = objcclass external;
  NSScanner = objcclass external;
  NSScriptCommandDescription = objcclass external;
  NSScriptClassDescription = objcclass external;
  NSScriptCoercionHandler = objcclass external;
  NSScriptCommand = objcclass external;
  NSScriptExecutionContext = objcclass external;
  NSScriptWhoseTest = objcclass external;
  NSIndexSpecifier = objcclass external;
  NSMiddleSpecifier = objcclass external;
  NSNameSpecifier = objcclass external;
  NSPositionalSpecifier = objcclass external;
  NSPropertySpecifier = objcclass external;
  NSRandomSpecifier = objcclass external;
  NSRangeSpecifier = objcclass external;
  NSRelativeSpecifier = objcclass external;
  NSUniqueIDSpecifier = objcclass external;
  NSWhoseSpecifier = objcclass external;
  NSCloneCommand = objcclass external;
  NSCloseCommand = objcclass external;
  NSCountCommand = objcclass external;
  NSCreateCommand = objcclass external;
  NSDeleteCommand = objcclass external;
  NSExistsCommand = objcclass external;
  NSGetCommand = objcclass external;
  NSMoveCommand = objcclass external;
  NSQuitCommand = objcclass external;
  NSSetCommand = objcclass external;
  NSMutableSet = objcclass external;
  NSScriptSuiteRegistry = objcclass external;
  NSLogicalTest = objcclass external;
  NSSpecifierTest = objcclass external;
  NSCountedSet = objcclass external;
  NSSortDescriptor = objcclass external;
  NSSpellServer = objcclass external;
  NSStream = objcclass external;
  NSMutableString = objcclass external;
  NSSimpleCString = objcclass external;
  NSConstantString = objcclass external;
  NSTask = objcclass external;
  NSTextCheckingResult = objcclass external;
  NSThread = objcclass external;
  NSUndoManager = objcclass external;
  NSURLAuthenticationChallenge = objcclass external;
  NSURLCredential = objcclass external;
  NSURLProtectionSpace = objcclass external;
  NSURLResponse = objcclass external;
  NSURLAuthenticationChallengeInternal = objcclass external;
  NSCachedURLResponseInternal = objcclass external;
  NSURLRequest = objcclass external;
  NSCachedURLResponse = objcclass external;
  NSURLCacheInternal = objcclass external;
  NSURLCache = objcclass external;
  NSURLConnectionInternal = objcclass external;
  NSURLConnection = objcclass external;
  NSURLCredentialInternal = objcclass external;
  NSURLCredentialStorageInternal = objcclass external;
  NSURLCredentialStorage = objcclass external;
  NSURLDownloadInternal = objcclass external;
  NSURLDownload = objcclass external;
  NSURLHandle = objcclass external;
  NSURLProtectionSpaceInternal = objcclass external;
  NSMutableURLRequest = objcclass external;
  NSURLProtocol = objcclass external;
  NSURLProtocolInternal = objcclass external;
  NSURLRequestInternal = objcclass external;
  NSURLResponseInternal = objcclass external;
  NSHTTPURLResponseInternal = objcclass external;
  NSHTTPURLResponse = objcclass external;
  NSUserDefaults = objcclass external;
  NSValue = objcclass external;
  NSValueTransformer = objcclass external;
  NSXMLDTD = objcclass external;
  NSXMLDocument = objcclass external;
  NSXMLDTDNode = objcclass external;
  NSXMLElement = objcclass external;
  NSXMLNode = objcclass external;
  NSXMLParser = objcclass external;
  NSImage = objcclass external;
  DOMDocument = objcclass external;
  DOMAbstractView = objcclass external;
  DOMCSSStyleDeclaration = objcclass external;
  DOMElement = objcclass external;
  DOMAttr = objcclass external;
  DOMBlob = objcclass external;
  DOMCDATASection = objcclass external;
  DOMCharacterData = objcclass external;
  DOMComment = objcclass external;
  DOMCounter = objcclass external;
  DOMCSSCharsetRule = objcclass external;
  DOMCSSFontFaceRule = objcclass external;
  DOMCSSStyleSheet = objcclass external;
  DOMMediaList = objcclass external;
  DOMCSSImportRule = objcclass external;
  DOMCSSRuleList = objcclass external;
  DOMCSSMediaRule = objcclass external;
  DOMCSSPageRule = objcclass external;
  DOMRGBColor = objcclass external;
  DOMRect = objcclass external;
  DOMCSSPrimitiveValue = objcclass external;
  DOMCSSRule = objcclass external;
  DOMCSSValue = objcclass external;
  DOMCSSStyleRule = objcclass external;
  DOMCSSUnknownRule = objcclass external;
  DOMCSSValueList = objcclass external;
  DOMDocumentFragment = objcclass external;
  DOMDocumentType = objcclass external;
  DOMEntityReference = objcclass external;
  DOMEvent = objcclass external;
  DOMHTMLCollection = objcclass external;
  DOMHTMLElement = objcclass external;
  DOMImplementation = objcclass external;
  DOMNode = objcclass external;
  DOMNodeIterator = objcclass external;
  DOMNodeList = objcclass external;
  DOMProcessingInstruction = objcclass external;
  DOMRange = objcclass external;
  DOMStyleSheetList = objcclass external;
  DOMText = objcclass external;
  DOMTreeWalker = objcclass external;
  DOMXPathExpression = objcclass external;
  DOMXPathResult = objcclass external;
  DOMNamedNodeMap = objcclass external;
  DOMEntity = objcclass external;
  DOMFile = objcclass external;
  DOMFileList = objcclass external;
  DOMHTMLAnchorElement = objcclass external;
  DOMHTMLAppletElement = objcclass external;
  DOMHTMLAreaElement = objcclass external;
  DOMHTMLBaseElement = objcclass external;
  DOMHTMLBaseFontElement = objcclass external;
  DOMHTMLBodyElement = objcclass external;
  DOMHTMLBRElement = objcclass external;
  DOMHTMLFormElement = objcclass external;
  DOMHTMLButtonElement = objcclass external;
  DOMHTMLDirectoryElement = objcclass external;
  DOMHTMLDivElement = objcclass external;
  DOMHTMLDListElement = objcclass external;
  DOMHTMLDocument = objcclass external;
  DOMHTMLEmbedElement = objcclass external;
  DOMHTMLFieldSetElement = objcclass external;
  DOMHTMLFontElement = objcclass external;
  DOMHTMLFrameElement = objcclass external;
  DOMHTMLFrameSetElement = objcclass external;
  DOMHTMLHeadElement = objcclass external;
  DOMHTMLHeadingElement = objcclass external;
  DOMHTMLHRElement = objcclass external;
  DOMHTMLHtmlElement = objcclass external;
  DOMHTMLIFrameElement = objcclass external;
  DOMHTMLImageElement = objcclass external;
  DOMHTMLInputElement = objcclass external;
  DOMHTMLIsIndexElement = objcclass external;
  DOMHTMLLabelElement = objcclass external;
  DOMHTMLLegendElement = objcclass external;
  DOMHTMLLIElement = objcclass external;
  DOMStyleSheet = objcclass external;
  DOMHTMLLinkElement = objcclass external;
  DOMHTMLMapElement = objcclass external;
  DOMHTMLMarqueeElement = objcclass external;
  DOMHTMLMenuElement = objcclass external;
  DOMHTMLMetaElement = objcclass external;
  DOMHTMLModElement = objcclass external;
  DOMHTMLObjectElement = objcclass external;
  DOMHTMLOListElement = objcclass external;
  DOMHTMLOptGroupElement = objcclass external;
  DOMHTMLOptionElement = objcclass external;
  DOMHTMLOptionsCollection = objcclass external;
  DOMHTMLParagraphElement = objcclass external;
  DOMHTMLParamElement = objcclass external;
  DOMHTMLPreElement = objcclass external;
  DOMHTMLQuoteElement = objcclass external;
  DOMHTMLScriptElement = objcclass external;
  DOMHTMLSelectElement = objcclass external;
  DOMHTMLStyleElement = objcclass external;
  DOMHTMLTableCaptionElement = objcclass external;
  DOMHTMLTableCellElement = objcclass external;
  DOMHTMLTableColElement = objcclass external;
  DOMHTMLTableSectionElement = objcclass external;
  DOMHTMLTableElement = objcclass external;
  DOMHTMLTableRowElement = objcclass external;
  DOMHTMLTextAreaElement = objcclass external;
  DOMHTMLTitleElement = objcclass external;
  DOMHTMLUListElement = objcclass external;
  DOMKeyboardEvent = objcclass external;
  DOMMouseEvent = objcclass external;
  DOMMutationEvent = objcclass external;
  DOMNotation = objcclass external;
  DOMObject = objcclass external;
  DOMOverflowEvent = objcclass external;
  DOMProgressEvent = objcclass external;
  NSColor = objcclass external;
  DOMUIEvent = objcclass external;
  DOMWheelEvent = objcclass external;
  WebView = objcclass external;
  WebArchivePrivate = objcclass external;
  WebResource = objcclass external;
  WebArchive = objcclass external;
  WebHistoryItem = objcclass external;
  WebBackForwardListPrivate = objcclass external;
  WebBackForwardList = objcclass external;
  WebDataSourcePrivate = objcclass external;
  WebFrame = objcclass external;
  WebDataSource = objcclass external;
  WebDownloadInternal = objcclass external;
  WebDownload = objcclass external;
  WebFramePrivate = objcclass external;
  WebFrameView = objcclass external;
  WebScriptObject = objcclass external;
  WebFrameViewPrivate = objcclass external;
  WebHistoryPrivate = objcclass external;
  WebHistory = objcclass external;
  WebHistoryItemPrivate = objcclass external;
  WebPolicyPrivate = objcclass external;
  WebPreferencesPrivate = objcclass external;
  WebPreferences = objcclass external;
  WebResourcePrivate = objcclass external;
  WebScriptObjectPrivate = objcclass external;
  WebUndefined = objcclass external;
  WebViewPrivate = objcclass external;

implementation

end.
