{ Source provided for Free Pascal Bug Report 3968 }
{ Submitted by "C Western" on  2005-05-14 }
{ e-mail: mftq75@dsl.pipex.com }
unit uw3968;
interface
type
 Tabcde = (a=0,b,c,d,e);
 Tabc = a..c;
implementation
end.
