{ Parsed from Quartzcore }
type
  NSPredicateOperator = objcclass external;
  NSHTTPCookieInternal = objcclass external;
  NSHTTPCookieStorageInternal = objcclass external;
  NSURLAuthenticationChallengeInternal = objcclass external;
  NSCachedURLResponseInternal = objcclass external;
  NSURLCacheInternal = objcclass external;
  NSURLConnectionInternal = objcclass external;
  NSURLCredentialInternal = objcclass external;
  NSURLCredentialStorageInternal = objcclass external;
  NSURLDownloadInternal = objcclass external;
  NSURLProtectionSpaceInternal = objcclass external;
  NSURLProtocolInternal = objcclass external;
  NSURLRequestInternal = objcclass external;
  NSURLResponseInternal = objcclass external;
  NSHTTPURLResponseInternal = objcclass external;
  _NSImageAuxiliary = objcclass external;
  NSTabWell = objcclass external;
  NSStorage = objcclass external;
  NSRunStorage = objcclass external;
  NSSortedArray = objcclass external;
  QTMovie = objcclass external;
  NSIBObjectData = objcclass external;
  NSManagedObjectContext = objcclass external;
  NSFetchRequest = objcclass external;
  NSMouseTracker = objcclass external;
  NSManagedObjectModel = objcclass external;
  NSPredicateTemplate = objcclass external;
  NSEntityDescription = objcclass external;
  NSNavView = objcclass external;
  NSSavePanelAuxiliary = objcclass external;
  NSToolbarItemViewer = objcclass external;
  _NSViewAuxiliary = objcclass external;
  NSWindowAuxiliary = objcclass external;
  CALayerArray = objcclass external;
