{ Old file: tbs0003.pp }
{  dito                                                OK 0.9.2 }

unit tb0002;

  interface

  implementation


    procedure dump_stack(bp : longint);

      function get_next_frame(bp : longint) : longint;

        begin
        end;

      begin
      end;

end.
